/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim: set ts=2 et sw=2 tw=80: */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

[scriptable, function, uuid(3a75d80a-9258-4ab8-95fd-ec0b5f634df1)]
interface nsICursorContinueCallback : nsISupports
{
  void handleContinue();
};

[scriptable, builtinclass, uuid(062ea35a-5158-425a-b7bc-3ae9daa84398)]
interface nsIDOMDOMCursor : nsISupports
{
  readonly attribute boolean done;
  void continue();
};
